@@ -1,22 +1,24 @@
+# Licensed under the LGPL: https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html
+# For details: https://github.com/PyCQA/astroid/blob/main/LICENSE
+# Copyright (c) https://github.com/PyCQA/astroid/blob/main/CONTRIBUTORS.txt
+
 import ast
 import sys
+import types
 from collections import namedtuple
 from functools import partial
-from typing import Optional
+from typing import Dict, Optional
 
-import astroid
+from astroid.const import PY38_PLUS, Context
 
-try:
-    import typed_ast.ast3 as _ast_py3
-except ImportError:
-    _ast_py3 = None
-
-
-PY38 = sys.version_info[:2] >= (3, 8)
-if PY38:
+if sys.version_info >= (3, 8):
     # On Python 3.8, typed_ast was merged back into `ast`
-    _ast_py3 = ast
-
+    _ast_py3: Optional[types.ModuleType] = ast
+else:
+    try:
+        import typed_ast.ast3 as _ast_py3
+    except ImportError:
+        _ast_py3 = None
 
 FunctionType = namedtuple("FunctionType", ["argtypes", "returns"])
 
@@ -36,7 +38,7 @@ class ParserModule(
 ):
     def parse(self, string: str, type_comments=True):
         if self.module is _ast_py3:
-            if PY38:
+            if PY38_PLUS:
                 parse_func = partial(self.module.parse, type_comments=type_comments)
             else:
                 parse_func = partial(
@@ -52,16 +54,14 @@ def parse_function_type_comment(type_comment: str) -> Optional[FunctionType]:
     if _ast_py3 is None:
         return None
 
-    func_type = _ast_py3.parse(type_comment, "<type_comment>", "func_type")
+    func_type = _ast_py3.parse(type_comment, "<type_comment>", "func_type")  # type: ignore[attr-defined]
     return FunctionType(argtypes=func_type.argtypes, returns=func_type.returns)
 
 
 def get_parser_module(type_comments=True) -> ParserModule:
-    if not type_comments:
-        parser_module = ast
-    else:
+    parser_module = ast
+    if type_comments and _ast_py3:
         parser_module = _ast_py3
-    parser_module = parser_module or ast
 
     unary_op_classes = _unary_operators_from_module(parser_module)
     cmp_op_classes = _compare_operators_from_module(parser_module)
@@ -121,10 +121,10 @@ def _compare_operators_from_module(module):
     }
 
 
-def _contexts_from_module(module):
+def _contexts_from_module(module) -> Dict[ast.expr_context, Context]:
     return {
-        module.Load: astroid.Load,
-        module.Store: astroid.Store,
-        module.Del: astroid.Del,
-        module.Param: astroid.Store,
+        module.Load: Context.Load,
+        module.Store: Context.Store,
+        module.Del: Context.Del,
+        module.Param: Context.Store,
     }
